"""
.. module:: keylayouts
   :synopsis: A class representing the iso keylayout.

.. moduleauthor:: Andrew J. Young

"""


# Imports from third party packages.
from typing import Dict, List, Union

# Imports from this package.
from symboard.keylayouts.keylayouts import Keylayout


class IsoKeylayout(Keylayout):
    """ An implementation of the iso keyboard layout (keylayout) which
    is able to produce an installable iso keyboard.
    """

    _DEFAULT_NAME: str = 'Iso keyboard'
    name: str = _DEFAULT_NAME

    default_index: int = 6

    def __str__(self):
        return 'IsoKeylayout({}, (id: {}))'.format(self.name, self.id_)

    def __init__(
        self,
        group: int,
        id_: int,
        maxout: int = 1,
        name: str = _DEFAULT_NAME,
        default_index: int = 0
    ):
        super(IsoKeylayout, self).__init__(
            group, id_, maxout, name=name, default_index=default_index
        )

    layouts: List[Dict[str, str]] = [
        {
            'first': '0',
            'last': '17',
            'mapSet': 'ANSI', # Yes, this is correct for iso too!
            'modifiers': 'Modifiers',
        },
    ]

    key_map_select: Dict[int, str] = {
        0: '',
        1: 'anyShift',
        2: 'anyOption',
        3: 'anyShift caps? anyOption',
        4: 'command',
        5: 'caps',
        6: 'anyControl',
    }

    key_map: Dict[int, Dict[int, str]] = {
        0: {
            0: 'a',
            1: 's',
            2: 'd',
            3: 'f',
            4: 'h',
            5: 'g',
            6: 'z',
            7: 'x',
            8: 'c',
            9: 'v',
            10: '',
            11: 'b',
            12: 'q',
            13: 'w',
            14: 'e',
            15: 'r',
            16: 'y',
            17: 't',
            18: '1',
            19: '2',
            20: '3',
            21: '4',
            22: '6',
            23: '5',
            24: '=',
            25: '9',
            26: '7',
            27: '-',
            28: '8',
            29: '0',
            30: ']',
            31: 'o',
            32: 'u',
            33: '[',
            34: 'i',
            35: 'p',
            36: '&#x000D;',
            37: 'l',
            38: 'j',
            39: '&#x0027;',
            40: 'k',
            41: ';',
            42: '\\',
            43: ',',
            44: '/',
            45: 'n',
            46: 'm',
            47: '.',
            48: '&#x0009;',
            49: ' ',
            50: '`',
            51: '&#x0008;',
            53: '&#x001B;',
            57: '',
            64: '&#x0010;',
            65: '',
            66: '&#x001D;',
            67: '*',
            69: '+',
            70: '&#x001C;',
            71: '&#x001B;',
            72: '&#x001F;',
            75: '/',
            76: '&#x0003;',
            77: '&#x001E;',
            78: '-',
            79: '&#x0010;',
            80: '&#x0010;',
            81: '=',
            82: '0',
            83: '1',
            84: '2',
            85: '3',
            86: '4',
            87: '5',
            88: '6',
            89: '7',
            91: '8',
            92: '9',
            96: '&#x0010;',
            97: '&#x0010;',
            98: '&#x0010;',
            99: '&#x0010;',
            100: '&#x0010;',
            101: '&#x0010;',
            103: '&#x0010;',
            105: '&#x0010;',
            106: '&#x0010;',
            107: '&#x0010;',
            109: '&#x0010;',
            111: '&#x0010;',
            113: '&#x0010;',
            114: '&#x0005;',
            115: '&#x0001;',
            116: '&#x000B;',
            117: '&#x007F;',
            118: '&#x0010;',
            119: '&#x0004;',
            120: '&#x0010;',
            121: '&#x000C;',
            122: '&#x0010;',
            123: '&#x001C;',
            124: '&#x001D;',
            125: '&#x001F;',
            126: '&#x001E;',
        },
        1: {
            0: 'A',
            1: 'S',
            2: 'D',
            3: 'F',
            4: 'H',
            5: 'G',
            6: 'Z',
            7: 'X',
            8: 'C',
            9: 'V',
            11: 'B',
            12: 'Q',
            13: 'W',
            14: 'E',
            15: 'R',
            16: 'Y',
            17: 'T',
            18: '!',
            19: '@',
            20: '#',
            21: '$',
            22: '^',
            23: '%',
            24: '+',
            25: '(',
            26: '&#x0026;',
            27: '_',
            28: '*',
            29: ')',
            30: '}',
            31: 'O',
            32: 'U',
            33: '{',
            34: 'I',
            35: 'P',
            36: '&#x000D;',
            37: 'L',
            38: 'J',
            39: '&#x0022;',
            40: 'K',
            41: ':',
            42: '|',
            43: '&#x003C;',
            44: '?',
            45: 'N',
            46: 'M',
            47: '&#x003E;',
            48: '&#x0009;',
            49: ' ',
            50: '~',
            51: '&#x0008;',
            53: '&#x001B;',
            64: '&#x0010;',
            66: '&#x001D;',
            67: '*',
            69: '+',
            70: '&#x001C;',
            71: '&#x001B;',
            72: '&#x001F;',
            75: '/',
            76: '&#x0003;',
            77: '&#x001E;',
            78: '-',
            79: '&#x0010;',
            80: '&#x0010;',
            81: '=',
            82: '0',
            83: '1',
            84: '2',
            85: '3',
            86: '4',
            87: '5',
            88: '6',
            89: '7',
            91: '8',
            92: '9',
            96: '&#x0010;',
            97: '&#x0010;',
            98: '&#x0010;',
            99: '&#x0010;',
            100: '&#x0010;',
            101: '&#x0010;',
            103: '&#x0010;',
            105: '&#x0010;',
            106: '&#x0010;',
            107: '&#x0010;',
            109: '&#x0010;',
            111: '&#x0010;',
            113: '&#x0010;',
            114: '&#x0005;',
            115: '&#x0001;',
            116: '&#x000B;',
            117: '&#x007F;',
            118: '&#x0010;',
            119: '&#x0004;',
            120: '&#x0010;',
            121: '&#x000C;',
            122: '&#x0010;',
            123: '&#x001C;',
            124: '&#x001D;',
            125: '&#x001F;',
            126: '&#x001E;',
        },
        2: {
            0: '',
            36: '&#x000D;',
            48: '&#x0009;',
            51: '&#x0008;',
            53: '&#x001B;',
            64: '&#x0010;',
            66: '&#x001D;',
            70: '&#x001C;',
            71: '&#x001B;',
            72: '&#x001F;',
            76: '&#x0003;',
            77: '&#x001E;',
            79: '&#x0010;',
            80: '&#x0010;',
            96: '&#x0010;',
            97: '&#x0010;',
            98: '&#x0010;',
            99: '&#x0010;',
            100: '&#x0010;',
            101: '&#x0010;',
            103: '&#x0010;',
            105: '&#x0010;',
            106: '&#x0010;',
            107: '&#x0010;',
            109: '&#x0010;',
            111: '&#x0010;',
            113: '&#x0010;',
            114: '&#x0005;',
            115: '&#x0001;',
            116: '&#x000B;',
            117: '&#x007F;',
            118: '&#x0010;',
            119: '&#x0004;',
            120: '&#x0010;',
            121: '&#x000C;',
            122: '&#x0010;',
            123: '&#x001C;',
            124: '&#x001D;',
            125: '&#x001F;',
            126: '&#x001E;',
        },
        3: {
            0: '',
            36: '&#x000D;',
            48: '&#x0009;',
            51: '&#x0008;',
            53: '&#x001B;',
            64: '&#x0010;',
            66: '&#x001D;',
            70: '&#x001C;',
            71: '&#x001B;',
            72: '&#x001F;',
            76: '&#x0003;',
            77: '&#x001E;',
            79: '&#x0010;',
            80: '&#x0010;',
            96: '&#x0010;',
            97: '&#x0010;',
            98: '&#x0010;',
            99: '&#x0010;',
            100: '&#x0010;',
            101: '&#x0010;',
            103: '&#x0010;',
            105: '&#x0010;',
            106: '&#x0010;',
            107: '&#x0010;',
            109: '&#x0010;',
            111: '&#x0010;',
            113: '&#x0010;',
            114: '&#x0005;',
            115: '&#x0001;',
            116: '&#x000B;',
            117: '&#x007F;',
            118: '&#x0010;',
            119: '&#x0004;',
            120: '&#x0010;',
            121: '&#x000C;',
            122: '&#x0010;',
            123: '&#x001C;',
            124: '&#x001D;',
            125: '&#x001F;',
            126: '&#x001E;',
        },
        4: {
            0: 'a',
            1: 's',
            2: 'd',
            3: 'f',
            4: 'h',
            5: 'g',
            6: 'z',
            7: 'x',
            8: 'c',
            9: 'v',
            11: 'b',
            12: 'q',
            13: 'w',
            14: 'e',
            15: 'r',
            16: 'y',
            17: 't',
            18: '1',
            19: '2',
            20: '3',
            21: '4',
            22: '6',
            23: '5',
            24: '=',
            25: '9',
            26: '7',
            27: '-',
            28: '8',
            29: '0',
            30: ']',
            31: 'o',
            32: 'u',
            33: '[',
            34: 'i',
            35: 'p',
            36: '&#x000D;',
            37: 'l',
            38: 'j',
            39: '&#x0027;',
            40: 'k',
            41: ';',
            42: '\\',
            43: ',',
            44: '/',
            45: 'n',
            46: 'm',
            47: '.',
            48: '&#x0009;',
            49: ' ',
            50: '`',
            51: '&#x0008;',
            53: '&#x001B;',
            64: '&#x0010;',
            66: '&#x001D;',
            67: '*',
            69: '+',
            70: '&#x001C;',
            71: '&#x001B;',
            72: '&#x001F;',
            75: '/',
            76: '&#x0003;',
            77: '&#x001E;',
            78: '-',
            79: '&#x0010;',
            80: '&#x0010;',
            81: '=',
            82: '0',
            83: '1',
            84: '2',
            85: '3',
            86: '4',
            87: '5',
            88: '6',
            89: '7',
            91: '8',
            92: '9',
            96: '&#x0010;',
            97: '&#x0010;',
            98: '&#x0010;',
            99: '&#x0010;',
            100: '&#x0010;',
            101: '&#x0010;',
            103: '&#x0010;',
            105: '&#x0010;',
            106: '&#x0010;',
            107: '&#x0010;',
            109: '&#x0010;',
            111: '&#x0010;',
            113: '&#x0010;',
            114: '&#x0005;',
            115: '&#x0001;',
            116: '&#x000B;',
            117: '&#x007F;',
            118: '&#x0010;',
            119: '&#x0004;',
            120: '&#x0010;',
            121: '&#x000C;',
            122: '&#x0010;',
            123: '&#x001C;',
            124: '&#x001D;',
            125: '&#x001F;',
            126: '&#x001E;',
        },
        5: {
            0: 'A',
            1: 'S',
            2: 'D',
            3: 'F',
            4: 'H',
            5: 'G',
            6: 'Z',
            7: 'X',
            8: 'C',
            9: 'V',
            11: 'B',
            12: 'Q',
            13: 'W',
            14: 'E',
            15: 'R',
            16: 'Y',
            17: 'T',
            18: '1',
            19: '2',
            20: '3',
            21: '4',
            22: '6',
            23: '5',
            24: '=',
            25: '9',
            26: '7',
            27: '-',
            28: '8',
            29: '0',
            30: ']',
            31: 'O',
            32: 'U',
            33: '[',
            34: 'I',
            35: 'P',
            36: '&#x000D;',
            37: 'L',
            38: 'J',
            39: '&#x0027;',
            40: 'K',
            41: ';',
            42: '\\',
            43: ',',
            44: '/',
            45: 'N',
            46: 'M',
            47: '.',
            48: '&#x0009;',
            49: ' ',
            50: '`',
            51: '&#x0008;',
            53: '&#x001B;',
            64: '&#x0010;',
            66: '&#x001D;',
            67: '*',
            69: '+',
            70: '&#x001C;',
            71: '&#x001B;',
            72: '&#x001F;',
            75: '/',
            76: '&#x0003;',
            77: '&#x001E;',
            78: '-',
            79: '&#x0010;',
            80: '&#x0010;',
            81: '=',
            82: '0',
            83: '1',
            84: '2',
            85: '3',
            86: '4',
            87: '5',
            88: '6',
            89: '7',
            91: '8',
            92: '9',
            96: '&#x0010;',
            97: '&#x0010;',
            98: '&#x0010;',
            99: '&#x0010;',
            100: '&#x0010;',
            101: '&#x0010;',
            103: '&#x0010;',
            105: '&#x0010;',
            106: '&#x0010;',
            107: '&#x0010;',
            109: '&#x0010;',
            111: '&#x0010;',
            113: '&#x0010;',
            114: '&#x0005;',
            115: '&#x0001;',
            116: '&#x000B;',
            117: '&#x007F;',
            118: '&#x0010;',
            119: '&#x0004;',
            120: '&#x0010;',
            121: '&#x000C;',
            122: '&#x0010;',
            123: '&#x001C;',
            124: '&#x001D;',
            125: '&#x001F;',
            126: '&#x001E;',
        },
        6: {
            0: '',
            36: '&#x000D;',
            48: '&#x0009;',
            51: '&#x0008;',
            53: '&#x001B;',
            64: '&#x0010;',
            66: '&#x001D;',
            70: '&#x001C;',
            71: '&#x001B;',
            72: '&#x001F;',
            76: '&#x0003;',
            77: '&#x001E;',
            79: '&#x0010;',
            80: '&#x0010;',
            96: '&#x0010;',
            97: '&#x0010;',
            98: '&#x0010;',
            99: '&#x0010;',
            100: '&#x0010;',
            101: '&#x0010;',
            103: '&#x0010;',
            105: '&#x0010;',
            106: '&#x0010;',
            107: '&#x0010;',
            109: '&#x0010;',
            111: '&#x0010;',
            113: '&#x0010;',
            114: '&#x0005;',
            115: '&#x0001;',
            116: '&#x000B;',
            117: '&#x007F;',
            118: '&#x0010;',
            119: '&#x0004;',
            120: '&#x0010;',
            121: '&#x000C;',
            122: '&#x0010;',
            123: '&#x001C;',
            124: '&#x001D;',
            125: '&#x001F;',
            126: '&#x001E;',
        },
    }
